# netfrog. A distributed platform to manage with frog data.
# Copyright (C) 2004 David Soulayrol <dsoulayrol@free.fr>
#
# This file is part of netfrog.
#
# netfrog is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#TODO : rename this module
"""Common objects to all NetFrog related developpment.
[TODO: to be wrote]
"""

# Default port for the server's listening socket
DEFAULT_PORT = 8881

# Error quotes from server
# TODO: to be defined
ERROR_AUTHENTICATION = '101 Authentication failed\n'

from ground.xml.parsers import XmlBootstrapParser
from ground.xml.consumers import SAXConsumer, RootSAXConsumer, Element2Field
from ground.util.log import Logger

# For introspection
import netfrog.server.services as services

import imp, os, string, socket


class ConfigurationException(Exception):
    """An exception related to configuration parsing.
    """



class Machine(object):
    """A wrapper for machine information.

    A Machine is actually a machine running a NetFrog server. Each
    machine must be unique by its name, IP address and eventually its
    port.
    """

    def __init__(self, aAddress, aPort = DEFAULT_PORT, aName = ''):
        """The unique constructor of a machine.
        """
        self.ip = aAddress
        self.port = aPort
        self.name = aName

    def __str__(self):
        """Returns a fully informative string for this machine.
        """
        return self.name + '(' + self.ip + ':' + str(self.port) + ')'

    def asTuple(self):
        return (self.ip, self.port)
    
    def getMachineFromAddress(aAddress, aPort = DEFAULT_PORT):
        lTuple = socket.gethostbyaddr(aAddress)
        return Machine(aAddress = lTuple[2][0],
                      aName = lTuple[0],
                      aPort = aPort)
    getMachineFromAddress = staticmethod(getMachineFromAddress)

    def getMachineFromString(aString):
        lName, lPort = aString.split(':')
        return Machine.getMachineFromAddress(lName, lPort)
    getMachineFromString = staticmethod(getMachineFromString)

    def getMachineFromPair(aPair):
        lName, lPort = aPair
        return Machine.getMachineFromAddress(lName, lPort)
    getMachineFromPair = staticmethod(getMachineFromPair)



class ServiceOption(Element2Field):
    def __init__(self, aService, aParameterName):
        Element2Field.__init__(self, self.setValue)
        self.__service = aService
        self.__name = aParameterName

    def setValue(self, aValue):
        self.__service.setParameter(self.__name, aValue)



class Service(SAXConsumer):
    def __init__(self, aName, aQName, aAttributes):
        SAXConsumer.__init__(self)
        Logger.info('New service %s created', aName)
        self.__moduleName = aName
        self.__name = self.__buildServiceName(aName)
        self.__parameters = self.__getDictionnary('__parameters')
        self.__commands = self.__getDictionnary('__commands')


    def getName(self):
        return self.__name


    def getModuleName(self):
        return self.__moduleName

    
    def getParameters(self):
        return self.__parameters
    

    def getCommands(self):
        return self.__commands

    
    def setParameter(self, aName, aValue):
        if aName in self.__parameters.keys():
            Logger.info('Parameter %s from Service %s updated with %s ',
                        aName, self.__name, aValue)
            self.__parameters[aName] = aValue
        else:
            raise ConfigurationException(
                'Service ' + self.__service + ' does not have '
                + aName + ' parameter')

        
    def consumeElement(self, aName, aQName, aAttributes):
        Logger.debug('Service.consumeElement(%s, %s, %s)',
                     aName, aQName, aAttributes)
        if aName in self.__parameters.keys():
            return ServiceOption(self, aName)
        else:
            raise ConfigurationException(
                'Service ' + self.__service + ' does not support '
                + aName + ' attribute')


    def consumeCharacters(self, aContent):
        raise ConfigurationException(
            'Element service does not consume characters')


    def __buildServiceName(self, aName):
        """Transform the element name into the related service name.
        """
        return 'Service' + aName.capitalize()
        
        
    def __getDictionnary(self, aName):
        """Return a dictionnary of the service related to this object.
        """
        lFile, lPath, lHelp = imp.find_module(
            self.__moduleName, services.__path__)
        lModule = imp.load_module(self.__moduleName, lFile, lPath, lHelp)
        lClass = lModule.__dict__.get(self.__name)
        return lClass.__dict__.get('_' + self.__name + aName)

        
class ServerConfiguration(RootSAXConsumer):
    PARSING_UNDONE = 0
    PARSING_ROOT = 1
    PARSING_SERVICES = 2
    PARSING_USERS = 3
    PARSING_DONE = 100


    def __init__(self, aFilename):
        RootSAXConsumer.__init__(self)
        self.__filename = aFilename
        self.__port = DEFAULT_PORT
	self.__services = {}
        self.__users = []
        self.__serviceNames = self.__buildServicesList()
        self.__parsing = ServerConfiguration.PARSING_UNDONE


    def setPort(self, aPort):
        self.__port = aPort


    def getPort(self):
        return self.__port


    def getFilename(self):
        return self.__filename


    def getServices(self):
        return self.__services
    

    def addAuthorizedUser(self, aUser):
        self.__users.append(aUser)


    def getAuthorizedUsers(self):
        return self.__users


    def consumeElement(self, aName, aQName, aAttributes):
        Logger.debug('ServerConfiguration.consumeElement(%s, %s, %s)',
                     aName, aQName, aAttributes)        
        if aName == u'services':
            if self.__parsing != ServerConfiguration.PARSING_ROOT:
                raise ConfigurationException(
                    'Misplaced element <services>')
            self.__parsing = ServerConfiguration.PARSING_SERVICES
            return self
    
	if aName in self.__serviceNames:
            return self.consumeServiceElement(aName, aQName, aAttributes)

        if aName == u'users':
            if self.__parsing != ServerConfiguration.PARSING_ROOT:
                raise ConfigurationException(
                    'Misplaced element <users>')
            self.__parsing = ServerConfiguration.PARSING_USERS
            return self
            
        if aName == u'user':
            if self.__parsing != ServerConfiguration.PARSING_USERS:
                raise ConfigurationException(
                    'Misplaced element <user>')
            return Element2Field(self.addAuthorizedUser)

        raise ConfigurationException('Unknown element <' + aName + '>')


    def consumeEndElement(self, aName, aQName):
        if aName == u'services':
            if self.__parsing != ServerConfiguration.PARSING_SERVICES:
                raise ConfigurationException(
                    'Misplaced element </services>')
            self.__parsing = ServerConfiguration.PARSING_ROOT
    
        elif aName == u'users':
            if self.__parsing != ServerConfiguration.PARSING_USERS:
                raise ConfigurationException(
                    'Misplaced element </users>')
            self.__parsing = ServerConfiguration.PARSING_ROOT
        

    def consumeServiceElement(self, aName, aQName, aAttributes):
        if self.__parsing != ServerConfiguration.PARSING_SERVICES:
            raise ConfigurationException(
                'Misplaced element <' + aName + '>')
        if self.__services.has_key(aName) == True:
            raise ConfigurationException(
                'Service <' + aName + '> declared twice')
        
        lService = Service(aName, aQName, aAttributes)
        self.__services[aName] = lService
        return lService

        
    def consumeCharacters(self, aContent):
        raise ConfigurationException(
            'Element server does not consume characters')


    def buildFromElement(self, aName, aQName, aAttributes):
        if self.__parsing != ServerConfiguration.PARSING_UNDONE:
            raise ConfigurationException(
                'Parsing was interrupted by a new tentative')
            
        if aName != u'server':
            raise ConfigurationException(
                'Root element should be <server>, not ' + aName)
            
        for lQName in aAttributes.getQNames():
            if lQName == 'port':
                self.__port = string.atoi(
                    aAttributes.getValueByQName(lQName))
            else:
                raise ConfigurationException(
                    'Unknown attribute ' + lQname + ' in' + aName)

        self.__parsing = ServerConfiguration.PARSING_ROOT
        Logger.debug('ServerConfiguration.buildFromElement(%s, %s, %s)',
                     aName, aQName, aAttributes)


    def __buildServicesList(self):
        """Build a list with all the available service names by introspection.
        """
        lServices = filter(
            lambda x: x.endswith('.py') and x not in ['__init__.py'],
            os.listdir(services.__path__[0]))
        return [x.split('.')[0] for x in lServices]
        
                
class ConfigurationParser(XmlBootstrapParser):
    def __init__(self, aServerConfiguration):
        XmlBootstrapParser.__init__(self,
                                    aServerConfiguration,
                                    aReportEndElements = True)

