# netfrog. A distributed platform to manage with frog data.
# Copyright (C) 2004 David Soulayrol <dsoulayrol@free.fr>
#
# This file is part of netfrog.
#
# netfrog is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, zipfile
import time, datetime

import ground.util.log as log

from netfrog.server.services import model

class ServiceStorage(model.AbstractService):
    LOCATION = 'location'
    CMD_NEW_DATA = 'storage-data'
    CMD_REPOSITORY = 'storage-repository'
    
    __parameters = { LOCATION: None }
    __commands = [ CMD_NEW_DATA, CMD_REPOSITORY ]
    
    def __init__(self, aUser, aServiceParameters):
        self.__file = None
        self.__fileTimestamp = None
        self.__user = aUser
        self.__root = os.path.join(aServiceParameters[self.LOCATION],
                                   self.__user.name)

        # Prepare the roots
        if not os.path.isdir(self.__root):
            os.makedirs(self.__root)
        os.chdir(self.__root)
        self.__checkFile(self.__getWeekTimeTuple())
        log.Logger.info('Service storage initialized for %s(%s)' % \
                        (self.__user, self.__root))


    def process(self, aMessage):
        lCommand = aMessage[:aMessage.index(' ')]
        lData = aMessage[aMessage.index(' ') + 1:]
        if lCommand.strip() == self.CMD_NEW_DATA:
            self.storeData(lData)
        if lCommand.strip() == self.CMD_REPOSITORY:
            # TODO: introduce return send back data to the emitter.
            # return __file ... name() ?
            pass

            
    def storeData(self, aFrogData):
        """Write the data to a file.
        
        Format is dd/MM/yyyy:hh/mm:temp:pres:hygro:[comment]
        """
        
        lData = ''
        lComment = ''
        lTimeTuple = self.__getTimeTuple()
        self.__checkFile(lTimeTuple)

        try:
            lData, lComment = aFrogData.split(':')
        except ValueError:
            # There was no symbol ':'.
            lData = aFrogData
        
        lTemperature, lPressure, lHygroscopy = lData.split(' ')
        log.Logger.info('%sdeg, %smBar, %s%% from %s' % \
                        (lTemperature, lPressure, lHygroscopy,
                         str(self.__user)))
        
        lLine = self.__formatDate(lTimeTuple) + ':' \
                + lTemperature + ':' + lPressure + ':' + lHygroscopy \
                + ':' + lComment.replace(' ', '%20') + os.linesep
        
        self.__file.write(lLine)
        self.__file.flush()
        

    def __getWeekTimeTuple(self):
        """Returns the 9-tuple of the beginning of this week.
        """
        lNowTuple = self.__getTimeTuple()
        if lNowTuple.tm_wday == 6:
            return lNowTuple
        
        lDelta = datetime.timedelta(days = - (lNowTuple.tm_wday + 2))
        lWeekDate = datetime.date.today() + lDelta
        return lWeekDate.timetuple()     


    def __getTimeTuple(self):
        """Returns the 9-tuple of this very moment.
        """
        return time.localtime(time.time())     


    def __formatDate(self, aTimeTuple):
        """Returns the date formatted to fit an entry in data files.
        """
        return time.strftime('%d/%m/%Y:%H/%M', aTimeTuple)

    def __formatFilename(self, aTimeTuple):
        """Returns data filename for the week beginning at the given date.
        """
        lFilename = time.strftime('%Y%m%d', aTimeTuple)
        return self.__user.name + '-' + lFilename

    def __checkFile(self, aTimeTuple):
        """Checks if a new file must be opened.

        The data from one user is kept in files under
        DATA_ROOT/user_name. Each file holds the data produced during
        one week (from saturday to friday midnight). The filenames are
        named after the first day of the week they represent (ie
        'machine-20030401'). Old data files are zipped.

        """
        if self.__user.name == '':
            log.Logger.warn('__checkFile: Internal Error:' + \
                               'no user authenticated.')
            return
            
        if self.__file == None:
            lFilename = self.__formatFilename(aTimeTuple)
            self.__file = open(lFilename, 'a')
            self.__fileTimestamp = aTimeTuple
            log.Logger.info('Logging %s to %s' % (self.__user, lFilename))
            return

        if aTimeTuple.tm_wday == 6 and \
           aTimeTuple.tm_mday != self.__fileTimestamp.tm_mday:

             # If this day is a new saturday, we must zip the file...
            print "DEBUG ** new saturday..."
            self.__file.close()
            try:
                lZipName = self.__file.name + '.zip'
                lZipFile = zipfile.ZipFile(lZipName, 'w')
                lZipFile.write(self.__file.name)
                lZipFile.close()
                log.Logger.info('Week data zipped in ' + lZipName)
            except OSError, e:
                log.Logger.warn('Error ', e)
            
            # ... and begin a new one 
            lFilename = self.__formatFilename(aTimeTuple)
            self.__file = open(lFilename, 'w')
            self.__fileTimestamp = aTimeTuple
            log.Logger.info('Now logging %s to %s' % \
                            (self.__user, lFilename))


